@@ -1,8 +1,7 @@
 """Main Nvim interface."""
-import functools
 import os
 import sys
-
+from functools import partial
 from traceback import format_stack
 
 from msgpack import ExtType
@@ -37,6 +36,12 @@ class Nvim(object):
     `SubClass.from_nvim(nvim)` where `SubClass` is a subclass of `Nvim`, which
     is useful for having multiple `Nvim` objects that behave differently
     without one affecting the other.
+
+    When this library is used on python3.4+, asyncio event loop is guaranteed
+    to be used. It is available as the "loop" attribute of this class. Note
+    that asyncio callbacks cannot make blocking requests, which includes
+    accessing state-dependent attributes. They should instead schedule another
+    callback using nvim.async_call, which will not have this restriction.
     """
 
     @classmethod
@@ -79,7 +84,7 @@ class Nvim(object):
         self.types = types
         self.api = RemoteApi(self, 'nvim_')
         self.vars = RemoteMap(self, 'nvim_get_var', 'nvim_set_var')
-        self.vvars = RemoteMap(self, 'vim_get_vvar', None)
+        self.vvars = RemoteMap(self, 'nvim_get_vvar', None)
         self.options = RemoteMap(self, 'nvim_get_option', 'nvim_set_option')
         self.buffers = Buffers(self)
         self.windows = RemoteSequence(self, 'nvim_list_wins')
@@ -91,6 +96,10 @@ class Nvim(object):
         self._decode = decode
         self._err_cb = err_cb
 
+        # only on python3.4+ we expose asyncio
+        if IS_PYTHON3 and os.name != 'nt':
+            self.loop = self._session.loop._loop
+
     def _from_nvim(self, obj, decode=None):
         if decode is None:
             decode = self._decode
@@ -181,6 +190,21 @@ class Nvim(object):
         """Stop the event loop being started with `run_loop`."""
         self._session.stop()
 
+    def close(self):
+        """Close the nvim session and release its resources."""
+        self._session.close()
+
+    def __enter__(self):
+        """Enter nvim session as a context manager."""
+        return self
+
+    def __exit__(self, *exc_info):
+        """Exit nvim session as a context manager.
+
+        Closes the event loop.
+        """
+        self.close()
+
     def with_decode(self, decode=True):
         """Initialize a new Nvim instance."""
         return Nvim(self._session, self.channel_id,
@@ -295,12 +319,12 @@ class Nvim(object):
 
         The returned sequences can be used as input to `feedkeys`.
         """
-        return self.request('vim_replace_termcodes', string,
+        return self.request('nvim_replace_termcodes', string,
                             from_part, do_lt, special)
 
-    def out_write(self, msg):
+    def out_write(self, msg, **kwargs):
         """Print `msg` as a normal message."""
-        return self.request('nvim_out_write', msg)
+        return self.request('nvim_out_write', msg, **kwargs)
 
     def err_write(self, msg, **kwargs):
         """Print `msg` as an error message."""
@@ -353,7 +377,7 @@ class Buffers(object):
     """Remote NVim buffers.
 
     Currently the interface for interacting with remote NVim buffers is the
-    `vim_get_buffers` msgpack-rpc function. Most methods fetch the list of
+    `nvim_list_bufs` msgpack-rpc function. Most methods fetch the list of
     buffers from NVim.
 
     Conforms to *python-buffers*.
@@ -440,7 +464,7 @@ class Funcs(object):
         self._nvim = nvim
 
     def __getattr__(self, name):
-        return functools.partial(self._nvim.call, name)
+        return partial(self._nvim.call, name)
 
 
 class NvimError(Exception):
